##        Copyright (c) 2000 by Harry Henry Gebel

##        Permission is hereby granted, free of charge, to any person
##        obtaining a copy of this software and associated documentation files
##        (the "Software"), to deal in the Software without restriction,
##        including without limitation the rights to use, copy, modify, merge,
##        publish, distribute, sublicense, and/or sell copies of the Software,
##        and to permit persons to whom the Software is furnished to do so,
##        subject to the following conditions:

##        The above copyright notice and this permission notice shall be
##        included in all copies or substantial portions of the Software.

##        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
##        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
##        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
##        NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
##        BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
##        ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
##        CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
##        SOFTWARE.

from ncurses import _curses # import curses interface

# place constants in local namespace
from _curses import TRUE, FALSE, COLOR_BLACK, COLOR_RED, COLOR_GREEN, \
     COLOR_YELLOW, COLOR_BLUE, COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE, \
     _SUBWIN, _ENDLINE, _FULLWIN, _SCROLLWIN, _ISPAD, _HASMOVED, _WRAPPED, \
     CCHARW_MAX, A_NORMAL, A_PROTECT, A_HORIZONTAL, A_LEFT, A_LOW, A_RIGHT, \
     A_TOP, A_VERTICAL, KEY_CODE_YES, KEY_MIN, KEY_BREAK, KEY_DOWN, KEY_UP, \
     KEY_LEFT, KEY_RIGHT, KEY_HOME, KEY_BACKSPACE, KEY_F0, KEY_DL, KEY_IL, \
     KEY_DC, KEY_IC, KEY_EIC, KEY_CLEAR, KEY_EOS, KEY_EOL, KEY_SF, KEY_SR, \
     KEY_NPAGE, KEY_PPAGE, KEY_STAB, KEY_CTAB, KEY_CATAB, KEY_ENTER, \
     KEY_SRESET, KEY_RESET, KEY_PRINT, KEY_LL, KEY_A1, KEY_A3, KEY_B2, \
     KEY_C1, KEY_C3, KEY_BTAB, KEY_BEG, KEY_CANCEL, KEY_CLOSE, KEY_COMMAND, \
     KEY_COPY, KEY_CREATE, KEY_END, KEY_EXIT, KEY_FIND, KEY_HELP, KEY_MARK, \
     KEY_MESSAGE, KEY_MOVE, KEY_NEXT, KEY_OPEN, KEY_OPTIONS, KEY_PREVIOUS, \
     KEY_REDO, KEY_REFERENCE, KEY_REFRESH, KEY_REPLACE, KEY_RESTART, \
     KEY_RESUME, KEY_SAVE, KEY_SBEG, KEY_SCANCEL, KEY_SCOMMAND, KEY_SCOPY, \
     KEY_SCREATE, KEY_SDC, KEY_SDL, KEY_SELECT, KEY_SEND, KEY_SEOL, \
     KEY_SEXIT, KEY_SFIND, KEY_SHELP, KEY_SHOME, KEY_SIC, KEY_SLEFT, \
     KEY_SMESSAGE, KEY_SMOVE, KEY_SNEXT, KEY_SOPTIONS, KEY_SPREVIOUS, \
     KEY_SPRINT, KEY_SREDO, KEY_SREPLACE, KEY_SRIGHT, KEY_SRSUME, KEY_SSAVE, \
     KEY_SSUSPEND, KEY_SUNDO, KEY_SUSPEND, KEY_UNDO, KEY_MOUSE, KEY_RESIZE, \
     KEY_MAX, NCURSES_MOUSE_VERSION, BUTTON1_RELEASED, BUTTON1_PRESSED, \
     BUTTON1_CLICKED, BUTTON1_DOUBLE_CLICKED, BUTTON1_TRIPLE_CLICKED, \
     BUTTON1_RESERVED_EVENT, BUTTON2_RELEASED, BUTTON2_PRESSED, \
     BUTTON2_CLICKED, BUTTON2_DOUBLE_CLICKED, BUTTON2_TRIPLE_CLICKED, \
     BUTTON2_RESERVED_EVENT, BUTTON3_RELEASED, BUTTON3_PRESSED, \
     BUTTON3_CLICKED, BUTTON3_DOUBLE_CLICKED, BUTTON3_TRIPLE_CLICKED, \
     BUTTON3_RESERVED_EVENT, BUTTON4_RELEASED, BUTTON4_PRESSED, \
     BUTTON4_CLICKED, BUTTON4_DOUBLE_CLICKED, BUTTON4_TRIPLE_CLICKED, \
     BUTTON4_RESERVED_EVENT, BUTTON_CTRL, BUTTON_SHIFT, BUTTON_ALT, \
     ALL_MOUSE_EVENTS, REPORT_MOUSE_POSITION, TRACE_DISABLE, TRACE_TIMES, \
     TRACE_TPUTS, TRACE_UPDATE, TRACE_MOVE, TRACE_CHARPUT, TRACE_ORDINARY, \
     TRACE_CALLS, TRACE_VIRTPUT, TRACE_IEVENT, TRACE_BITS, TRACE_ICALLS, \
     TRACE_CCALLS, TRACE_MAXIMUM, OPTIMIZE_MVCUR, OPTIMIZE_HASHMAP, \
     OPTIMIZE_SCROLL, OPTIMIZE_ALL, E_OK, E_SYSTEM_ERROR, E_BAD_ARGUMENT, \
     E_POSTED, E_CONNECTED, E_BAD_STATE, E_NO_ROOM, E_NOT_POSTED, \
     E_UNKNOWN_COMMAND, E_NO_MATCH, E_NOT_SELECTABLE, E_NOT_CONNECTED, \
     E_REQUEST_DENIED, E_INVALID_FIELD, E_CURRENT, A_ATTRIBUTES, A_CHARTEXT, \
     A_COLOR, A_STANDOUT, A_UNDERLINE, A_REVERSE, A_BLINK, A_DIM, A_BOLD, \
     A_ALTCHARSET, A_INVIS

# import some built-in functions
from _curses import keybound, curses_version, define_key, keyok, \
     resizeterm, use_default_colors, baudrate, beep, can_change_color, \
     cbreak, curs_set, def_prog_mode, def_shell_mode, \
     delay_output, doupdate, echo, endwin, erasechar, filter, \
     flash, flushinp, halfdelay, has_colors, has_ic, has_il, init_color, \
     init_pair, isendwin, keyname, killchar, longname, napms, \
     nl, nocbreak, noecho, nonl, noqiflush, noraw, \
     qiflush, raw, resetty, reset_prog_mode, reset_shell_mode, savetty, \
     scr_dump, scr_init, scr_restore, scr_set, slk_attroff, \
     slk_attron, slk_attrset, slk_attr, slk_attr_set, slk_clear, slk_color, \
     slk_init, slk_label, slk_noutrefresh, slk_refresh, slk_restore, slk_set, \
     slk_touch, start_color, termattrs, termname, \
     typeahead, ungetch, use_env, \
     mcprint, has_key, _nc_tracebits, _tracechar, \
     trace, _nc_visbuf

# some functions to access global variables (including acs_map)

def LINES():
  return _curses.cursesvar.LINES

def COLS():
  return _curses.cursesvar.COLS

def TABSIZE():
  return _curses.cursesvar.TABSIZE

def COLORS():
  return _curses.cursesvar.COLORS

def COLOR_PAIRS():
  return _curses.cursesvar.COLOR_PAIRS

def ACS_ULCORNER():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('l'))

def ACS_LLCORNER():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('m'))

def ACS_URCORNER():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('k'))

def ACS_LRCORNER():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('j'))

def ACS_LTEE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('t'))

def ACS_RTEE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('u'))

def ACS_BTEE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('v'))

def ACS_TTEE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('w'))

def ACS_HLINE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('q'))

def ACS_VLINE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('x'))

def ACS_PLUS():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('n'))

def ACS_S1():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('o'))

def ACS_S9():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('s'))

def ACS_DIAMOND():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('`'))

def ACS_CKBOARD():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('a'))

def ACS_DEGREE():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('f'))

def ACS_PLMINUS():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('g'))

def ACS_BULLET():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('~'))

def ACS_LARROW():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord(','))

def ACS_RARROW():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('+'))

def ACS_DARROW():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('.'))

def ACS_UARROW():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('-'))

def ACS_BOARD():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('h'))

def ACS_LANTERN():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('i'))

def ACS_BLOCK():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('0'))

def ACS_S3():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('p'))

def ACS_S7():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('r'))

def ACS_LEQUAL():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('y'))

def ACS_GEQUAL():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('z'))

def ACS_PI():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('{'))

def ACS_NEQUAL():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('|'))

def ACS_STERLING():
  return _curses.ptrvalue(_curses.cursesvar.acs_map, ord('}'))

def ACS_BSSB():
  return ACS_ULCORNER()

def ACS_SSBB():
  return ACS_LLCORNER()

def ACS_BBSS():
  return ACS_URCORNER()

def ACS_SBBS():
  return ACS_LRCORNER()

def ACS_SBSS():
  return ACS_RTEE()

def ACS_SSSB():
  return ACS_LTEE()

def ACS_SSBS():
  return ACS_BTEE()

def ACS_BSSS():
  return ACS_TTEE()

def ACS_BSBS():
  return ACS_HLINE()

def ACS_SBSB():
  return ACS_VLINE()

def ACS_SSSS():
  return ACS_PLUS()

# wrappers for a few functions

# returns a tuple (red, green, blue)
def color_content(color):
  (red_p, green_p, blue_p) = (_curses.ptrcreate('short'),
                              _curses.ptrcreate('short'),
                              _curses.ptrcreate('short'))
  _curses.color_content(color, red_p, green_p, blue_p)
  (red, green, blue) = (_curses.ptrvalue(red_p), _curses.ptrvalue(green_p),
                        _curses.ptrvalue(blue_p))
  _curses.ptrfree(red_p)
  _curses.ptrfree(green_p)
  _curses.ptrfree(blue_p)
  return (red, green, blue)

# returns a tuple (foreground, background)
def pair_content(pair):
  (foreground_p, background_p) = (_curses.ptrcreate('short'),
                                  _curses.ptrcreate('short'))
  _curses.color_content(pair, foreground_p, background_p)
  (foreground, background) = (_curses.ptrvalue(foreground_p),
                              _curses.ptrvalue(background_p))
  _curses.ptrfree(foreground_p)                              
  _curses.ptrfree(background_p)
  return (foreground, background)

def getch():
  return _curses.wgetch(_curses.cursesvar.stdscr)

def clear():
  """ Clears stdscr """
  return _curses.wclear(_curses.cursesvar.stdscr)


# the man pages state that the window argument is ignored, so place intrflush
# here
def intrflush(bf):
  return _curses.intrflush(_curses.cursesvar.stdscr, bf)

def keypad(bf):
  return _curses.keypad(_curses.cursesvar.stdscr, bf)

# Utility functions

# convert a string or tuple to a chtype
# strings must be one character long, tuples must have a single character
# string as the first argument and an integer as the second. 
def chtype(ch):
  if type(ch) == type(()): 
    if len(ch) != 2: # check arguments
      raise TypeError('Tuple must contain two elements')
    return ord(ch[0]) | ch[1] # combine arguments
  elif type(ch) == type(''):
    return ord(ch)
  else:
    return ch

# convert a chtype to a single character string
def chtype2chr(ch):
  return chr(ch & 0xFF)

# these funtions return WINDOW class instances

def getwin(a):
  return WINDOW(a, function = 'getwin')

def initscr():
  return WINDOW(function = 'initscr')

def newpad(a, b):
  return WINDOW(a, b, function = 'newpad')

def newwin(a, b, c, d):
  return WINDOW(a, b, c, d, 'newwin')


# class WINDOW - represents ncurses WINDOW objects
class WINDOW:
    # static variables
  initialized = FALSE # TRUE if initscr has been called
  
  def __init__(self, a = None, b = None, c = None, d = None,
               function = 'newwin'):
    if not WINDOW.initialized and function != 'initscr': 
      raise ncursesError('curses not initialized')
    if function == 'newwin':
      if d == None:
        raise TypeError('newwin requires four parameters')
      self.WINDOW = _curses.newwin(a, b, c, d)
    elif function == 'initscr':
      if WINDOW.initialized:
        raise TypeError('initscr already called')
      self.WINDOW = _curses.initscr()
      WINDOW.initialized = TRUE
    elif function == 'getwin':
      raise NotImplementedError('function getwin not yet implemented.')
    elif function == 'newpad':
      if b == None or c != None:
        raise TypeError('newpad requires two parameters')
      self.WINDOW = newpad(a, b)
    elif function == 'pointer': # used for derwin, dupwin, and subwin
      if a == None or b != None:
        raise TypeError('pointer requiers one parameter')
      self.WINDOW = a
    else:
      raise ncursesError('unknown initialization function: ' + function)
    self.user_data = None

  def __del__(self): # restore terminal when done
    if _curses.cursesvar.stdscr == self.WINDOW:
      endwin()
    _curses.delwin(self.WINDOW)

  # utility functions
  def __makeChArray(self, chstr):
    if type(chstr) == type(''):
      attrib = 0
    elif type(chstr) == type(()):
      if len(chstr) != 2:
        raise TypeError('Tuple must contain two elements')
      (chstr, attrib) = chstr[0], chstr[1]
    else:
      raise TypeError('Argmument must be string or tuple')
    chstr_p = _curses.ptrcreate('long', 0, len(chstr) + 1) # build array
    for ch in xrange(len(chstr)):
      _curses.ptrset(chstr_p, chtype((chstr[ch], attrib)), ch)
    return _curses.ptrcast(chstr_p, 'chtype *')    
    
  def resize(self, a, b):
    return _curses.wresize(self.WINDOW, a, b)

  def keypad(self, bf):
    return _curses.keypad(self.WINDOW, bf)

  def clearok(self, a):
    return _curses.clearok(self.WINDOW, a)

  def copywin(self, destwin, sminrow, smincol, dminrow, dmincol, dmaxrow,
              dmaxcol, overlay):
    return _curses.copywin(self.WINDOW, destwin.WINDOW, sminrow, smincol,
                           dminrow, dmincol, dmaxrow,
                           dmaxcol, overlay)

  def derwin(self, a, b, c, d):
    return WINDOW(_curses.derwin(self.WINDOW, a, b, c, d),
                  function = 'pointer')

  def dupwin(self):
    return WINDOW(_curses.dupwin(self.WINDOW), function = 'pointer')

  def idcok(self, a):
    return _curses.idcok(self.WINDOW, a)

  def idlok(self, a):
    return _curses.idlok(self.WINDOW, a)

  def immedok(self, a):
    return _curses.immedok(self.WINDOW, a)

  def is_linetouched(self, a):
    return _curses.is_linetouched(self.WINDOW, a)

  def is_wintouched(self):
    return _curses.is_wintouched(self.WINDOW)

  def leaveok(self, a):
    return _curses.leaveok(self.WINDOW, a)

  def meta(self, a):
    return _curses.meta(self.WINDOW, a)

  def mvderwin(self, a, b):
    return _curses.mvderwin(self.WINDOW, a, b)

  def mvwin(self, a, b):
    return _curses.mvwin(self.WINDOW, a, b)

  def nodelay(self, a):
    return _curses.nodelay(self.WINDOW, a)

  def notimeout(self, a):
    return _curses.notimeout(self.WINDOW, a)

  def overlay(self, dstwin):
    return _curses.overlay(self.WINDOW, dstwin.WINDOW)

  def overwrite(self, dstwin):
    return _curses.overwrite(self.WINDOW, dstwin.WINDOW)

  def pechochar(self, a):
    return _curses.pechochar(self.WINDOW, a)

  def pnoutrefresh(self, a, b, c, d, e, f):
    return _curses.pnoutrefresh(self.WINDOW, a, b, c, d, e, f)

  def prefresh(self, a, b, c, d, e, f):
    return _curses.prefresh(self.WINDOW, a, b, c, d, e, f)

  def putwin(self, a):
    raise NotImplementedError('"putwin" not implemented')

  def scrollok(self, a):
    return _curses.scrollok(self.WINDOW, a)

  def subpad(self, nlines, ncols, begin_y, begin_x):
    return WINDOW(_curses.subpad(self.WINDOW, nlines, ncols, begin_y, begin_x),
                  function = 'pointer')

  def subwin(self, a, b, c, d):
    return WINDOW(_curses.subwin(self.WINDOW, a, b, c, d),
                  function = 'pointer')

  def syncok(self, a):
    return _curses.syncok(self.WINDOW, a)

  # chstr must be either a single character string or a tuple containing a
  # single character string and an integer, or an integer
  def addch(self, ch):
    return _curses.waddch(self.WINDOW, chtype(ch))

  # chstr must be either a string or a tuple containing a string and an integer
  def addchnstr(self, chstr, n):
    chstr = self.__makeChArray(chstr)
    _curses.waddchnstr(self.WINDOW, chstr, n)
    _curses.ptrfree(chstr) # deallocate array

  # substitute for the very useful addchstr macro
  def addchstr(self, chstr, n = -1):
    self.addchnstr(chstr, n)

  def addnstr(self, str, n):
    _curses.waddnstr(self.WINDOW, str, n)

  # another macro substitute
  def addstr(self, str, n = -1):
    self.addnstr(str, n)

  def attr_on(self, attrs):
    return _curses.wattr_on(self.WINDOW, attrs, 'NULL')

  def attr_off(self, attrs):
    return _curses.wattr_off(self.WINDOW, attrs, 'NULL')

  # does this work??????
  def attrset(self, attrs):
    self.attr_off(A_ATTRIBUTES)
    return self.attr_on(attrs)

  def bkgd(self, ch): 
    return _curses.wbkgd(self.WINDOW, chtype(a))

  def bkgdset(self, ch):
    return _curses.wbkgdset(self.WINDOW, chtype(a))

  def border(self, ls, rs, ts, bs, tl, tr, bl, br):
    return _curses.wborder(self.WINDOW, chtype(ls), chtype(rs), chtype(ts),
                           chtype(bs), chtype(tl), chtype(tr), chtype(bl),
                           chtype(br))

  # yet another macro
  def box(self, verch, horch):
    return self.border(verch, verch, horch, horch, 0, 0, 0, 0)

  # draws a lined border around side
  def line_border(self):
    return self.border(0, 0, 0, 0, 0, 0, 0, 0)

  def chgat(self, n, attr, color):
    return _curses.wchgat(self.WINDOW, n, attr, color, 'NULL')

  def clear(self):
    return _curses.wclear(self.WINDOW)

  def clrtobot(self):
    return _curses.wclrtobot(self.WINDOW)

  def clrtoeol(self):
    return _curses.wclrtoeol(self.WINDOW)

  def color_set(self, color_pair_number):
    return _curses.wcolor_set(self.WINDOW, color_pair_number, 'NULL')

  def cursyncup(self):
    return _curses.wcursyncup(self.WINDOW)

  def delch(self):
    return _curses.wdelch(self.WINDOW)

  def echochar(self, ch):
    return _curses.wechochar(self.WINDOW, chtype(ch))

  def erase(self):
    return _curses.werase(self.WINDOW)

  def getch(self):
    return _curses.wgetch(self.WINDOW)

  # reads a string up to length n
  def getnstr(self, n):
    buffer = _curses.ptrcreate('char', '', n + 2)
    _curses.wgetnstr(self.WINDOW, buffer, n)
    string = _curses.ptrvalue(buffer)
    _curses.ptrfree(buffer)
    return string

  # macro
  def getstr(self, n = 80):
    return self.getnstr(n)

  def hline(self, ch, n):
    return _curses.whline(self.WINDOW, chtype(ch), n)

  def inch(self):
    return _curses.winch(self.WINDOW)

  # perform type cast before returning result of inch
  def inchCh(self):
    return chtype2chr(self.inch())

  # returns a list of chtypes
  def inchnstr(self, n):
    if n == -1:
       # make sure buffer is long enough
      buffer = _curses.ptrcreate('long', 0, COLS() + 1)
    else:
      buffer = _curses.ptrcreate('long', 0, n + 1) 
    buffer = _curses.ptrcast(buffer, 'chtype *')
    _curses.winchnstr(self.WINDOW, buffer, n) # read buffer
    buffer = _curses.ptrcast(buffer, 'long *')
    index = 0 # loop through buffer until null
    array = []
    value = _curses.ptrvalue(buffer, index)
    while value:
      array.append(value)
      index = index + 1
      value = _curses.ptrvalue(buffer, index)
    _curses.ptrfree(buffer)
    return array

  # macro
  def inchstr(self, n = -1):
    return self.inchnstr(n)

  def innstr(self, n):
    if n == -1:
       # make sure buffer is long enough
      buffer = _curses.ptrcreate('char', 0, COLS() + 2)
    else:
      buffer = _curses.ptrcreate('char', 0, n + 2)
    _curses.winnstr(self, buffer, n)
    string = _curses.ptrvalue(buffer)
    _curses.ptrfree(buffer)
    return string

  # macro
  def instr(self, n = -1):
    return self.innstr(n)

  def insch(self, ch):
    return _curses.winsch(self.WINDOW, chtype(ch))

  def insdelln(self, a):
    return _curses.winsdelln(self.WINDOW, a)

  def insnstr(self, str, n):
    return _curses.winsnstr(self.WINDOW, str, n)

  # macro
  def insstr(self, str, n = -1):
    return self.insnstr(str, n)

  def move(self, a, b):
    return _curses.wmove(self.WINDOW, a, b)

  def noutrefresh(self):
    return _curses.wnoutrefresh(self.WINDOW)

  def redrawln(self, a, b):
    return _curses.wredrawln(self.WINDOW, a, b)

  def refresh(self):
    return _curses.wrefresh(self.WINDOW)

  def scrl(self, a):
    return _curses.wscrl(self.WINDOW, a)

  def setscrreg(self, a, b):
    return _curses.wsetscrreg(self.WINDOW, a, b)

  def syncdown(self):
    return _curses.wsyncdown(self.WINDOW)

  def syncup(self):
    return _curses.wsyncup(self.WINDOW)

  def timeout(self, a):
    return _curses.wtimeout(self.WINDOW, a)

  def touchln(self, y, n, changed = 1):
    return _curses.wtouchln(self.WINDOW, y, n, changed)

  def vline(self, ch, n):
    return _curses.wvline(self.WINDOW, chtype(ch), n)

  def getyx(self):
    cury_p = _curses.ptrcreate('int')
    curx_p = _curses.ptrcreate('int')
    _curses.GetYX(self.WINDOW, cury_p, curx_p)
    cury = _curses.ptrvalue(cury_p)
    curx = _curses.ptrvalue(curx_p)
    _curses.ptrfree(cury_p)
    _curses.ptrfree(curx_p)
    return (cury,curx)

  def getmaxyx(self):
    maxy_p = _curses.ptrcreate('int')
    maxx_p = _curses.ptrcreate('int')
    _curses.GetMaxYX(self.WINDOW, maxy_p, maxx_p)
    maxy = _curses.ptrvalue(maxy_p)
    maxx = _curses.ptrvalue(maxx_p)
    _curses.ptrfree(maxy_p)
    _curses.ptrfree(maxx_p)
    return (maxy,maxx)

  def getbegyx(self):
    begy_p = _curses.ptrcreate('int')
    begx_p = _curses.ptrcreate('int')
    _curses.GetBegYX(self.WINDOW, begy_p, begx_p)
    begy = _curses.ptrvalue(begy_p)
    begx = _curses.ptrvalue(begx_p)
    _curses.ptrfree(begy_p)
    _curses.ptrfree(begx_p)
    return (begy, begx)

# Exceptions
class ncursesError(Exception):
  pass
