##        Copyright (c) 2000 by Harry Henry Gebel

##        Permission is hereby granted, free of charge, to any person
##        obtaining a copy of this software and associated documentation files
##        (the "Software"), to deal in the Software without restriction,
##        including without limitation the rights to use, copy, modify, merge,
##        publish, distribute, sublicense, and/or sell copies of the Software,
##        and to permit persons to whom the Software is furnished to do so,
##        subject to the following conditions:

##        The above copyright notice and this permission notice shall be
##        included in all copies or substantial portions of the Software.

##        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
##        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
##        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
##        NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
##        BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
##        ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
##        CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
##        SOFTWARE.

from ncurses.curses import *
from ncurses import _panel
from ncurses import _curses

# import built-in functions
from _panel import update_panels

# wrappers

def doupdate():
  update_panels()
  _curses.doupdate()

# new functions
def getTop(panels):
  top_panel = _panel.panel_below('NULL')
  for panel in panels:
    if panel.PANEL == top_panel:
      return panel
  # if function makes it through loop than the top panel was not in the list
  raise ncursesError('Top panel not in provided list')

def getBottom(panels):
  top_panel = _panel.panel_above('NULL')
  for panel in panels:
    if panel.PANEL == top_panel:
      return panel
  # if function makes it through loop than the bottom panel was not in the list
  raise ncursesError('Bottom panel not in provided list')

# PANEL class - not to be used with a window already part of a WINDOW class
# instance
class PANEL(WINDOW):
  def __init__(self, a = None, b = None, c = None, d = None,
               function = 'newwin'):
    WINDOW.__init__(self, a, b, c, d, function) # initialize window
    self.PANEL = _panel.new_panel(self.WINDOW) # attach panel

  def __del__(self):
    _panel.del_panel(self.PANEL)
    _curses.delwin(self.WINDOW)

  def bottom_panel(self):
    return _panel.bottom_panel(self.PANEL)
    
  def top_panel(self):
    return _panel.top_panel(self.PANEL)
  
  def show_panel(self):
    return _panel.show_panel(self.PANEL)
    
  def hide_panel(self):
    return _panel.hide_panel(self.PANEL)

  def replace_panel(self, window):
    (self.WINDOW, window.WINDOW) = (window.WINDOW, self.WINDOW)
    return _panel.replace_panel(self.PANEL, self.WINDOW)
  
  def move_panel(self, starty, startx):
    return _panel.move_panel(self.PANEL, starty, startx)
    
  def panel_hidden(self):
    return  _panel.panel_hidden(self.PANEL)

  def panel_above(self, panels):
    above = _panel.panel_above(self.PANEL)
    if above == 'NULL': # if I am the top panel
      return self
    for panel in panels:
      if panel.PANEL == above:
        return panel
    raise ncursesError('Panel not found in list')

  def panel_below(self, panels):
    below = _panel.panel_below(self.PANEL)
    if below == 'NULL': # if I am the bottom panel
      return below
    for panel in panels:
      if panel.PANEL == below:
        return panel
    raise ncursesError('Panel not found in list')
